from src.helpers import TimeTracking
import logging
import time
from src.data_generator import DataGenerator
from src.experiment_setup import ExperimentSetup
import argparse


class run(object):
    """
    basic run class to measure execution time. Either call this class calling it by 'with' or delete the class instance
    after finishing the measurement. The duration result is logged.
    """

    def __init__(self):
        self.time = TimeTracking()
        logging.info(f"{self.__class__.__name__} started")

    def __del__(self):
        self.time.stop()
        logging.info(f"{self.__class__.__name__} finished after {self.time}")

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    def do_stuff(self):
        time.sleep(2)


class PreProcessing(run):

    def __init__(self, setup):
        super().__init__()
        self.setup = setup
        self.kwargs = None
        self._run()

    def _run(self):
        self.kwargs = {'start': '1997-01-01', 'end': '2017-12-31', 'limit': 1, 'window_history': 13,
                            'window_lead_time': 3, 'method': 'linear',
              'statistics_per_var': self.setup.var_all_dict, }
        self.check_valid_stations()

    def check_valid_stations(self):
        t = TimeTracking
        logging.debug("check valid stations started")
        window_lead_time = self.kwargs.get("window_lead_time", None)
        valid_stations = []
        for s in self.setup.stations:
            valid = False
            args = self.setup.__dict__
            args["stations"] = s

            h = DataGenerator(**args, **self.kwargs)
            da_it = h.get_data_generator(s)
            print('hi')



class Training(run):

    def __init__(self, setup):
        super().__init__()
        self.setup = setup


class PostProcessing(run):

    def __init__(self, setup):
        super().__init__()
        self.setup = setup


if __name__ == "__main__":

    formatter = '%(asctime)s - %(levelname)s: %(message)s  [%(filename)s:%(funcName)s:%(lineno)s]'
    logging.basicConfig(format=formatter, level=logging.DEBUG)

    parser = argparse.ArgumentParser()
    parser.add_argument('--experiment_date', metavar='--exp_date', type=str, nargs=1, default=None,
                        help="set experiment date as string")
    args = parser.parse_args()
    with run():
        setup = ExperimentSetup(args, test=True)
        PreProcessing(setup)
